@@ -54,6 +54,10 @@
 #define ENETUNREACH   128       /* Interactive doesn't know this */
 #endif /* ENETUNREACH */
 
+#ifdef SSL_ENABLE
+#include <openssl/ssl.h>	/* for OPENSSL_NO_SSL2 and ..._SSL3 checks */
+#endif
+
 /* prototypes for internal functions */
 static int load_params(int, char **, int);
 static void dump_params (struct runctl *runp, struct query *, flag implicit);
@@ -72,6 +76,7 @@ flag versioninfo;	    /* emit only version info */
 char *user;		    /* the name of the invoking user */
 char *home;		    /* invoking user's home directory */
 char *fmhome;		    /* fetchmail's home directory */
+int at_home;	    	    /* fetchmail is running inside the user's home directory */
 const char *program_name;   /* the name to prefix error messages with */
 flag configdump;	    /* dump control blocks for configurator */
 const char *fetchmailhost;  /* either `localhost' or the host's FQDN */
@@ -137,8 +142,8 @@ static void printcopyright(FILE *fp) {
 	fprintf(fp, GT_("Copyright (C) 2002, 2003 Eric S. Raymond\n"
 		   "Copyright (C) 2004 Matthias Andree, Eric S. Raymond,\n"
 		   "                   Robert M. Funk, Graham Wilson\n"
-		   "Copyright (C) 2005 - 2006, 2010 - 2012 Sunil Shetye\n"
-		   "Copyright (C) 2005 - 2012 Matthias Andree\n"
+		   "Copyright (C) 2005 - 2012 Sunil Shetye\n"
+		   "Copyright (C) 2005 - 2019 Matthias Andree\n"
 		   ));
 	fprintf(fp, GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
 		   "are welcome to redistribute it under certain conditions. For details,\n"
@@ -155,6 +160,8 @@ int main(int argc, char **argv)
 {
     int bkgd = FALSE;
     int implicitmode = FALSE;
+    flag safewithbg = FALSE; /** if parsed options are compatible with a
+			      fetchmail copy running in the background */
     struct query *ctl;
     netrc_entry *netrc_list;
     char *netrc_file, *tmpbuf;
@@ -198,7 +205,7 @@ int main(int argc, char **argv)
 
 #define IDFILE_NAME	".fetchids"
     run.idfile = prependdir (IDFILE_NAME, fmhome);
-  
+
     outlevel = O_NORMAL;
 
     /*
@@ -211,18 +218,16 @@ int main(int argc, char **argv)
      */
     fm_lock_dispose();
 
-#ifdef HAVE_GETCWD
     /* save the current directory */
     if (getcwd (currentwd, sizeof (currentwd)) == NULL) {
 	report(stderr, GT_("could not get current working directory\n"));
 	currentwd[0] = 0;
     }
-#endif
 
     {
 	int i;
 
-	i = parsecmdline(argc, argv, &cmd_run, &cmd_opts);
+	i = parsecmdline(argc, argv, &cmd_run, &cmd_opts, &safewithbg);
 	if (i < 0)
 	    exit(PS_SYNTAX);
 
@@ -262,6 +267,10 @@ int main(int argc, char **argv)
 #endif /* ODMR_ENABLE */
 #ifdef SSL_ENABLE
 	"+SSL"
+	"-SSLv2"
+#if (HAVE_DECL_SSLV3_CLIENT_METHOD + 0 == 0) || defined(OPENSSL_NO_SSL3)
+	"-SSLv3"
+#endif
 #endif
 #ifdef OPIE_ENABLE
 	"+OPIE"
@@ -533,17 +542,23 @@ int main(int argc, char **argv)
 	else if (getpid() == pid)
 	    /* this test enables re-execing on a changed rcfile */
 	    fm_lock_assert();
-	else if (argc > 1)
+	else if (argc > 1 && !safewithbg)
 	{
 	    fprintf(stderr,
 		    GT_("fetchmail: can't accept options while a background fetchmail is running.\n"));
+	    {
+		int i;
+		fprintf(stderr, "argc = %d, arg list:\n", argc);
+		for (i = 1; i < argc; i++) fprintf(stderr, "arg %d = \"%s\"\n", i, argv[i]);
+	    }
 	    return(PS_EXCLUDE);
 	}
 	else if (kill(pid, SIGUSR1) == 0)
 	{
-	    fprintf(stderr,
-		    GT_("fetchmail: background fetchmail at %ld awakened.\n"),
-		    (long)pid);
+	    if (outlevel > O_SILENT)
+		fprintf(stderr,
+			GT_("fetchmail: background fetchmail at %ld awakened.\n"),
+			(long)pid);
 	    return(0);
 	}
 	else
@@ -685,11 +700,9 @@ int main(int argc, char **argv)
 	{
 	    report(stdout, GT_("restarting fetchmail (%s changed)\n"), rcfile);
 
-#ifdef HAVE_GETCWD
 	    /* restore the startup directory */
 	    if (!currentwd[0] || chdir (currentwd) == -1)
 		report(stderr, GT_("attempt to re-exec may fail as directory has not been restored\n"));
-#endif
 
 	    /*
 	     * Matthias Andree: Isn't this prone to introduction of
@@ -952,7 +965,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     list_merge(&h2->domainlist, &h1->domainlist, force);
     list_merge(&h2->antispam, &h1->antispam, force);
 
-#define FLAG_MERGE(fld) if (force ? !!h1->fld : !h2->fld) h2->fld = h1->fld
+#define FLAG_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) h2->fld = h1->fld; } while (0)
     FLAG_MERGE(server.via);
     FLAG_MERGE(server.protocol);
     FLAG_MERGE(server.service);
@@ -1035,6 +1048,8 @@ static int load_params(int argc, char **argv, int optind)
     struct query def_opts, *ctl;
     struct stat rcstat;
     char *p;
+    unsigned int mboxcount = 0;
+    unsigned int idlecount = 0;
 
     run.bouncemail = TRUE;
     run.softbounce = TRUE;	/* treat permanent errors as temporary */
@@ -1236,7 +1251,7 @@ static int load_params(int argc, char **argv, int optind)
 
 	/*
 	 * We no longer do DNS lookups at startup.
-	 * This is a kluge.  It enables users to edit their
+	 * This is a kludge.  It enables users to edit their
 	 * configurations when DNS isn't available.
 	 */
 	ctl->server.truename = xstrdup(ctl->server.queryname);
@@ -1258,7 +1273,7 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->server.dns, TRUE);
 	    DEFAULT(ctl->server.uidl, FALSE);
 	    DEFAULT(ctl->use_ssl, FALSE);
-	    DEFAULT(ctl->sslcertck, FALSE);
+	    DEFAULT(ctl->sslcertck, TRUE);
 	    DEFAULT(ctl->server.checkalias, FALSE);
 #ifndef SSL_ENABLE
 	    /*
@@ -1386,9 +1401,18 @@ static int load_params(int argc, char **argv, int optind)
 		(void) fprintf(stderr,
 			       GT_("Both fetchall and keep on in daemon or idle mode is a mistake!\n"));
 	    }
+
+	    if (ctl->idle) ++idlecount;
+	    mboxcount += count_list(&ctl->mailboxes);
 	}
     }
 
+    if (idlecount && mboxcount > 1) {
+	fprintf(stderr,
+		GT_("fetchmail: Error: idle mode does not work for multiple folders or accounts!\n"));
+	exit(PS_SYNTAX);
+    }
+
     /*
      * If the user didn't set a last-resort user to get misaddressed
      * multidrop mail, set an appropriate default here.
@@ -1498,7 +1522,7 @@ static int query_host(struct query *ctl)
 		st = query_host(ctl);
 	    } while 
 		(st == PS_REPOLL);
-	    if (st == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP || st == PS_MAXFETCH || st == PS_DNS)
+	    if (st == PS_SUCCESS || st == PS_NOMAIL || st == PS_SOCKET || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP || st == PS_MAXFETCH || st == PS_DNS)
 		break;
 	}
 	ctl->server.protocol = P_AUTO;
@@ -1552,6 +1576,7 @@ static int query_host(struct query *ctl)
     default:
 	report(stderr, GT_("unsupported protocol selected.\n"));
 	st = PS_PROTOCOL;
+	break;
     }
 
     /*
@@ -1570,6 +1595,14 @@ static int query_host(struct query *ctl)
     return(st);
 }
 
+static int print_id_of(struct uid_db_record *rec, void *unused)
+{
+    (void)unused;
+
+    printf("\t%s\n", rec->id);
+    return 0;
+}
+
 static void dump_params (struct runctl *runp,
 			 struct query *querylist, flag implicit)
 /* display query parameters in English */
@@ -1701,6 +1734,8 @@ static void dump_params (struct runctl *runp,
 	    printf(GT_("  SSL protocol: %s.\n"), ctl->sslproto);
 	if (ctl->sslcertck) {
 	    printf(GT_("  SSL server certificate checking enabled.\n"));
+	} else {
+	    printf(GT_("  SSL server certificate checking disabled.\n"));
 	}
 	if (ctl->sslcertfile != NULL)
 		printf(GT_("  SSL trusted certificate file: %s\n"), ctl->sslcertfile);
@@ -1971,20 +2006,14 @@ static void dump_params (struct runctl *runp,
 
 	if (ctl->server.protocol > P_POP2 && MAILBOX_PROTOCOL(ctl))
 	{
-	    if (!ctl->oldsaved)
+	    int count;
+
+	    if (!(count = uid_db_n_records(&ctl->oldsaved)))
 		printf(GT_("  No UIDs saved from this host.\n"));
 	    else
 	    {
-		struct idlist *idp;
-		int count = 0;
-
-		for (idp = ctl->oldsaved; idp; idp = idp->next)
-		    ++count;
-
 		printf(GT_("  %d UIDs saved.\n"), count);
-		if (outlevel >= O_VERBOSE)
-		    for (idp = ctl->oldsaved; idp; idp = idp->next)
-			printf("\t%s\n", idp->id);
+		traverse_uid_db(&ctl->oldsaved, print_id_of, NULL);
 	    }
 	}
 
